#include "renovation work.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <ranges>
#include <algorithm>
#include <iomanip>

work::work(const char* t_company, const char* t_work_type, const char* t_work_unit, double t_salary_work_unit,int t_day_end, double t_percentage_completed)
	:salary_work_unit(t_salary_work_unit)
	, day_end(t_day_end)
	, percentage_completed(t_percentage_completed)
{
	strcpy_s(company, 50, t_company);
	strcpy_s(work_type, 50, t_work_type);
	strcpy_s(work_unit, 50, t_work_unit);
}

std::istream& operator >> (std::istream& is, work& t_work)
{
	is.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	std::cout << "Name company: ";
	is.getline(t_work.company,50);
	std::cout << "Work type: ";
	is.getline(t_work.work_type,50);
	std::cout << "Work unit: ";
	is.getline(t_work.work_unit,50);
	std::cout << "Salary per work unit: ";
	is >> t_work.salary_work_unit;
	std::cout << "Day End Working: ";
	is >> t_work.day_end;
	std::cout << "Percentage completed: ";
	is >> t_work.percentage_completed;
	return is;
}

std::ostream& operator << (std::ostream& os, const work& t_work)
{
	os << t_work.company << std::endl << t_work.work_type << std::endl
		<< t_work.work_unit << std::endl << t_work.salary_work_unit << std::endl
		<< t_work.day_end << std::endl<< t_work.percentage_completed << std::endl;
	return os;
}


Record::Record(const std::string& r_name)
	:name_record(r_name)
{}

void Record::create_record() {
	std::ofstream ofstr(name_record, std::ios::binary);
	int option;
	work p;
	std::cout << "Enter the information of Renovation work (Enter the first line to start the input):" << std::endl;
	std::cout << "Name company - Work type - Work unit - Salary per work unit - Day End - Percentage completed" << std::endl;
	do {
		std::cin >> p;
		ofstr.write(reinterpret_cast<char*>(&p), sizeof(p));
		std::cout << "0 - END" << std::endl << "1 - CONTINUE" << std::endl << "Enter option: ";
		std::cin >> option;
		if (option < 0 && option > 1) {
			std::cout << "Not request like that" << std::endl;
			break;
		}
	} while (option == 1);
}

void Record::get_data()
{
	std::ifstream ifstr(name_record, std::ios::binary);
	work p;
	int idx = 0;
	while (ifstr.read(reinterpret_cast<char*>(&p), sizeof(p))) {
		record_map.insert(std::make_pair(++idx, p));
	}
}

void Record::add_to_record(const work& t_work) {
	std::ofstream ofstr(name_record, std::ios::binary | std::ios::app);
	ofstr.write(reinterpret_cast<const char*>(&t_work), sizeof(work));
}

void Record::add_to_record_list(const std::initializer_list<work>& p_list)
{
	std::ofstream ofstr(name_record, std::ios::binary | std::ios::app);
	for (const auto& ele : p_list)
		ofstr.write(reinterpret_cast<const char*>(&ele), sizeof(work));
}

void Record::delete_from_record(int idx)
{
	get_data();
	record_map.erase(idx);
	std::ofstream fout(name_record + "_updated", std::ios::binary);
	for (const auto& ele : record_map)
		fout.write(reinterpret_cast<const char*>(&ele.second), sizeof(work));
}

void Record::replace_in_record(int idx, const work& p) {
	get_data();
	std::ofstream fout(name_record + "_updated", std::ios::binary);
	record_map[idx] = p;
	for (const auto& ele : record_map)
		fout.write(reinterpret_cast<const char*>(&ele.second), sizeof(p));
}

void Record::transform_into_text() {
	get_data();
	std::ofstream fout(name_record + ".txt");
	fout << "------------------------------------------ Renovation work -------------------------------------------------" << std::endl;
	fout << "____________________________________________________________________________________________________________" << std::endl;
	fout << "| No |  Name company  |   Work type    |    Work unit   |  Salary/unit   |    Day End     |  % completed   |" << std::endl;
	fout << "|____|________________|________________|________________|________________|________________|________________|" << std::endl;
	size_t count = 0;
	for (const auto& ele : record_map) {
		count++;
		fout << "|" << std::setw(4) << count << "|"
			<< std::setw(16) << ele.second.company << "|"
			<< std::setw(16)  << ele.second.work_type << "|"
			<< std::setw(16)  << ele.second.work_unit << "|"
			<< std::setw(16)  << ele.second.salary_work_unit << "|"
			<< std::setw(16)  << ele.second.day_end << "|"
			<< std::setw(16) << ele.second.percentage_completed << "|"
			<< std::endl;
	}
	fout << "|____|________________|________________|________________|________________|________________|________________|" << std::endl;
}

void Record::search_by_company(const std::string& t_company)
{
	get_data();
	std::ofstream fout(t_company, std::ios::binary);
	std::vector<work> res;
	std::ranges::copy(
		record_map
		| std::ranges::views::transform([](const std::pair<int, work>& p) {return p.second; })
		| std::ranges::views::filter([&t_company](const work& elem) { return elem.company == t_company; })
		, std::back_inserter(res));

	for (const auto& e : res)
		fout.write(reinterpret_cast<const char*>(&e), sizeof(work));
}

void Record::search_by_work_type(const std::string& t_work_type) {
	get_data();
	std::ofstream fout(t_work_type, std::ios::binary);
	std::vector<work> res;
	std::ranges::copy(
		record_map
		| std::ranges::views::transform([](const std::pair<int, work>& p) {return p.second; })
		| std::ranges::views::filter([&t_work_type](const work& elem) { return elem.work_type == t_work_type; })
		, std::back_inserter(res));

	for (const auto& e : res)
		fout.write(reinterpret_cast<const char*>(&e), sizeof(work));
}

void Record::search_by_day_end(int t_day_end) {
	get_data();
	std::ofstream ofstr(std::to_string(t_day_end), std::ios::binary);
	std::vector<work> res;
	std::ranges::copy(
		record_map
		| std::ranges::views::transform([](const std::pair<int, work>& p) {return p.second; })
		| std::ranges::views::filter([&t_day_end](const work& ele) { return ele.day_end == t_day_end ; })
		, std::back_inserter(res));
	for (const auto& e : res)
		ofstr.write(reinterpret_cast<const char*>(&e), sizeof(work));
}








